//= require <dSS/data/ZoneStore>
//= require <dSS/data/Loader>
Ext.namespace('dSS');

dSS.ZoneView = Ext.extend(Ext.DataView, {
    initComponent: function() {

        var zoneTemplate = new Ext.XTemplate(
            '<tpl for=".">', 
                '<div class="zone-wrap {css}" id="zone-{id}">', 
                    '<span>', 
                        '{name}',
                        '<tpl if="id !== 0">',
                            ' #{id}',
                        '</tpl>',
                        '<tpl if="(unnamedDevices === true)">',
                            ' *',
                        '</tpl>',
                    '</span>', 
                '</div>', 
            '</tpl>', 
            '<div class="x-clear"></div>');

        Ext.apply(this, {
            store: dSS.data.Loader.getZoneStore(),
            tpl: zoneTemplate,
            singleSelect: true,
            multiSelect: false,
            layout: 'fit',
            style: "overflow: auto",
            overClass: 'x-view-over',
            itemSelector: 'div.zone-wrap',
            emptyText: dSS._("No rooms to display")
        });

        dSS.ZoneView.superclass.initComponent.apply(this, arguments);

        dSS.data.Loader.getDeviceStore().on('load', function() {
                if (this.getSelectionCount() === 0) {
                    this.select(this.store.getById(0));
                }
                this.updateNumDevicesHeader();
            },
            this);

        // Here you can add functionality that requires the object to
        // exist, like event handling.
        this.on('selectionchange', function(dv, nodes) {
            if (this.getSelectionCount() === 0) {
                this.select(this.store.getById(0));
            }
            this.filterDevices();
            this.updateNumDevicesHeader();
        },
        this);

        this.on('render', function() {
            var zoneView = this;
            this.dropZone = new Ext.dd.DropZone(zoneView.getEl(), {
                ddGroup: "zoneDeviceDD",
                containerScroll: true,
                getTargetFromEvent: function(e) {
                    return e.getTarget(zoneView.itemSelector);
                },
                onNodeEnter: function(target, dd, e, data) {
                    Ext.fly(target).addClass('my-row-highlight-class');
                },
                onNodeOut: function(target, dd, e, data) {
                    Ext.fly(target).removeClass('my-row-highlight-class');
                },
                onNodeOver: function(target, dd, e, data) {
                    var record = zoneView.getRecord(target);
                    if (record.get('id') === 0) {
                        return Ext.dd.DropZone.prototype.dropNotAllowed;
                    }
                    return Ext.dd.DropZone.prototype.dropAllowed;
                },
                onNodeDrop: function(target, dd, e, data) {
                    var record = zoneView.getRecord(target);
                    if (record.get('id') === 0) {
                        return false;
                    }

                    Ext.each(data.selections, function(device) {
                        var currentDevice = device;
                        Ext.Ajax.request({
                            url: '/json/structure/zoneAddDevice',
                            disableCaching: true,
                            method: "GET",
                            scope: zoneView,
                            params: {
                                deviceID: currentDevice.get('dsid'),
                                zone: record.get('id')
                            },
                            success: function(result, request) {
                                try {
                                    var jsonData = Ext.util.JSON.decode(result.responseText);
                                    if (jsonData.ok) {
                                        var oldz = currentDevice.get('zoneId');
                                        currentDevice.set("zoneId", record.data.id);
                                        currentDevice.store.commitChanges();
                                        this.filterDevices();
                                        this.updateNumDevicesHeader();
                                        dSS.MessageBus.fireEvent('scenesmarkdirty');
                                        dSS.data.Loader.getZoneStore().fireEvent(
                                            'zonedeviceleft', oldz);
                                        dSS.data.Loader.getZoneStore().fireEvent(
                                            'zonedevnamechanged',
                                            currentDevice);
                                    } else {
                                        Ext.Msg.alert(dSS._("Error"),
                                            dSS._("Device") +
                                            ' ' + dSS.util.shortDSID(
                                                    currentDevice.get('dsid')) +
                                            ' ' + dSS._("could not be moved") +
                                            ': ' + jsonData.message);
                                    }
                                }
                                catch(err) {
                                    Ext.Msg.alert(
                                        dSS._("Error"), dSS._("Device") + ' ' +
                                        dSS.util.shortDSID(
                                            currentDevice.get('dsid')) + ' ' +
                                        dSS._("could not be moved") +
                                        ': ' + err);
                                }
                            },
                            failure: function(result, request) {
                                Ext.Msg.alert(
                                    dSS._("Error"), dSS._("Device") +  ' ' +
                                    dSS.util.shortDSID(
                                        currentDevice.get('dsid')) + ' ' +
                                    dSS._("could not be moved") + ': ' + err);
                            }
                        });
                    });
                    return true;
                }
            });
        },
        this);

        this.on('containerclick', function() {
                if (this.getSelectionCount() === 0) {
                    this.select(this.store.getById(0));
                }
            },
            this);

        this.on('contextmenu', function(view, index, node, event) {
            event.preventDefault();
            event.stopEvent();
            if (!this.contextMenu) {
                this.contextMenu = new Ext.menu.Menu({});
            } else {
                this.contextMenu.removeAll();
            }

            this.select(index);

            var menuItem;

            menuItem = new Ext.menu.Item({
                text: dSS._("Blink"),
                iconCls: 'blink',
                handler: this.blinkZone.createDelegate(this, index, true)
            });
            this.contextMenu.add(menuItem);

            menuItem = new Ext.menu.Item({
                text: dSS._("Create new room"),
                iconCls: 'addZoneIcon',
                handler: this.createNewZone.createDelegate(this, index, true)
            });
            this.contextMenu.add(menuItem);

            menuItem = new Ext.menu.Item({
                text: dSS._("Rename room"),
                iconCls: 'editZoneIcon',
                handler: this.editZone.createDelegate(this, index, true)
            });
            this.contextMenu.add(menuItem);

            menuItem = new Ext.menu.Item({
                text: dSS._("Delete room"),
                iconCls: 'removeZoneIcon',
                handler: this.removeZone.createDelegate(this, index, true)
            });
            this.contextMenu.add(menuItem);


            var xy = event.getXY();
            this.contextMenu.showAt(xy);
        },
        this);
    },

    filterDevices: function() {
        var selectedZones = this.getSelectedRecords();
        var store = dSS.data.Loader.getDeviceStore();

        if (selectedZones.length === 0) {
            store.clearFilter();
            return;
        }

        zrecord = selectedZones[0]; // single select is on
        if (zrecord.get('id') === 0) {
            store.clearFilter();
            return;
        }

        store.filterBy(function(record) {
            var i;
            for (i = 0; i < selectedZones.length; i++) {
                if (record.get('zoneId') === selectedZones[i].get('id')) {
                    return true;
                }
            }
            return false;
        });
    },

    updateNumDevicesHeader: function() {
        var count = dSS.data.Loader.getDeviceStore().getRange().length;
        var selectedZones = this.getSelectedRecords();
        var title = '';
        if ((selectedZones.length === 0) ||
            (selectedZones[0].get('id') === 0)) {
            title = dSS._("Devices in all rooms");
        } else {
            title = dSS._("Devices in room") + " \"" + selectedZones[0].get('name') + " - " + selectedZones[0].get('id') + "\"";
        }
        this.findParentByType('dsszonebrowser').devicePanel.setTitle("<span class=\"x-panel-header-text\">" + title + "</span><span style=\"float: right;\" class=\"x-panel-header-text\">" + count + " " + dSS._("Device", "n% Devices", count) + "</span>");
    },

    removeZone: function(item, event, index) {
        var record;

        if (index !== undefined) {
            record = this.getStore().getAt(index);
        } else {
            var records = this.getSelectedRecords();
            if (records.length === 0) {
                Ext.Msg.alert(dSS._("Error"), dSS._("No room selected"));
                return;
            }

            record = records[0]; // single select is on
        }

        var deviceStore = dSS.data.Loader.getDeviceStore();

        if (deviceStore.getRange().length > 0) {
            Ext.Msg.alert(dSS._("Error"), dSS._("Can not delete a non empty room"));
            return;
        }

        Ext.Ajax.request({
            url: '/json/structure/removeZone',
            disableCaching: true,
            method: "GET",
            scope: this,
            params: {
                zoneID: record.get('id')
            },
            success: function(result, request) {
                try {
                    var jsonData = Ext.util.JSON.decode(result.responseText);
                    if (jsonData.ok) {
                        this.getStore().remove(record);
                    } else {
                        Ext.Msg.alert(dSS._("Error"),
                                     dSS._("Could not remove room") + ': ' +
                                     jsonData.message);
                    }
                } catch(error) {
                    Ext.Msg.alert(dSS._("Error"),
                                  dSS._("Could not remove room") + ': ' +
                                  error);
                }
            },
            failure: function(result, request) {
                Ext.Msg.alert(dSS._("Error"),
                              dSS._("Could not remove room"));
            }
        });
    },

    blinkZone: function(item, event, index) {
        var record;

        if (index !== undefined) {
            record = this.getStore().getAt(index);
        } else {
            var records = this.getSelectedRecords();
            if (records.length === 0) {
                Ext.Msg.alert(dSS._("Error"), dSS._("No room selected"));
                return;
            }

            record = records[0]; // single select is on
        }

        Ext.Ajax.request({
            url: '/json/zone/blink',
            disableCaching: true,
            method: "GET",
            scope: this,
            params: {
                id: record.get('id')
            },
            success: function(result, request) {
                try {
                    var jsonData = Ext.util.JSON.decode(result.responseText);
                    if (!jsonData.ok) {
                        Ext.MessageBox.alert(dSS._("Error"), dSS._("Could not blink room") + " \"" + record.get('name') + "\"");
                    }
                } catch(err) {
                    Ext.MessageBox.alert(dSS._("Error"), dSS._("Could not blink room") + " \"" + record.get('name') + "\"");
                }
            },
            failure: function(result, request) {
                Ext.MessageBox.alert(dSS._("Error"), dSS._("Could not blink room") + " \"" + record.get('name') + "\"");
            }
        });
    },
    editZone: function(item, event, index) {
        var record;

        if (index !== undefined) {
            record = this.getStore().getAt(index);
        } else {
            var records = this.getSelectedRecords();
            if (records.length === 0) {
                Ext.Msg.alert(dSS._("Error"), dSS._("No room selected"));
                return;
            }

            record = records[0]; // single select is on
        }

        if (record.get('id') === 0) {
            Ext.Msg.alert(dSS._("Error"),
                          dSS._("Renaming 'All devices' entry is not allowed"));
            return;
        }

        Ext.Msg.prompt(dSS._("Rename room"), dSS._("New name for room"), function(btn, text) {
            if (text !== record.get('name')) {
                Ext.Ajax.request({
                    url: '/json/zone/setName',
                    disableCaching: true,
                    method: "GET",
                    params: {
                        id: record.get('id'),
                        newName: text
                    },
                    success: function(result, request) {
                        try {
                            var jsonData = Ext.util.JSON.decode(result.responseText);
                            if (jsonData.ok) {
                                record.set('name', text);
                                record.commit();
                                var devStore = dSS.data.Loader.getDeviceStore();
                                devStore.each(
                                    function(devRecord) {
                                        if (devRecord.get('zoneId') ===
                                                record.get('id')) {
                                            devRecord.set('zoneName',
                                                        text);
                                            devRecord.commit();
                                        }
                                    }
                                );
                            } else {
                                Ext.Msg.alert(dSS._("Error"),
                                              dSS._("Could not rename room") +
                                              ': ' + jsonData.message);
                            }
                        }
                        catch(err) {
                            Ext.Msg.alert(dSS._("Error"),
                                          dSS._("Could not rename room") +
                                          ': ' + err);
                        }
                    },
                    failure: function(result, request) {
                        Ext.Msg.alert(dSS._("Error"), 
                                      dSS._("Could not rename room"));
                    }
              });
            }
        },
        this, false, record.get('name'));
    },
    createNewZone: function() {
        var zoneStore = this.getStore();
        Ext.Msg.prompt(dSS._("Create new room"), dSS._("Name for new room"), function(btn, text) {
            if (btn === 'ok') {
                var i;
                for (i = 0; i <= zoneStore.data.length; i++) {
                    if (zoneStore.findExact('id', i) === - 1) {

                        Ext.Ajax.request({
                            url: '/json/structure/addZone',
                            disableCaching: true,
                            method: "GET",
                            params: {
                                zoneID: i
                            },
                            success: function(result, request) {
                                try {
                                    var jsonData = Ext.util.JSON.decode(result.responseText);
                                    if (jsonData.ok) {

                                        Ext.Ajax.request({
                                            url: '/json/zone/setName',
                                            disableCaching: true,
                                            method: "GET",
                                            params: {
                                                id: i,
                                                newName: text
                                            },
                                            success: function(result, request) {
                                                try {
                                                    var jsonData = Ext.util.JSON.decode(result.responseText);
                                                    if (jsonData.ok) {
                                                        zoneStore.loadData({
                                                            zones: [
                                                            {
                                                                id: i,
                                                                name: text
                                                            }]
                                                        },
                                                        true);
                                                    } else {
                                                        Ext.MessageBox.alert(dSS._("Error"), dSS._("Could not create zone") + ": " + json.message);
                                                    }
                                                }
                                                catch(err) {
                                                    Ext.MessageBox.alert(dSS._("Error"), dSS._("Could not create zone") + ": " + err);
                                                }
                                            },
                                            failure: function(result, request) {
                                                Ext.MessageBox.alert(dSS._("Error"), dSS._("Could not create zone"));
                                            }
                                        });

                                    }
                                }
                                catch(err) {
                                    Ext.MessageBox.alert(dSS._("Error"), dSS._("Could not create zone"));
                                }
                            },
                            failure: function(result, request) {
                                Ext.MessageBox.alert(dSS._("Error"), dSS._("Could not create zone"));
                            }
                        });
                        return;
                    }
                }
            }
        });
    }
});

Ext.reg('dsszoneview', dSS.ZoneView);

