//= require <dSS/GetTextHelper>
//= require <dSS/config/Config>
Ext.namespace("dSS", "dSS.devices", "dSS.config");

// group
//  stop button will be shown depending on the group
dSS.devices.DeviceSettingsPanel = Ext.extend(Ext.Panel, {
    constructor: function(config) {

        this.dsid = config.dsid;

        this.valueField = new Ext.form.NumberField({
            x: 120,
            y: 12,
            width: 50,
            allowBlank: false,
            allowNegative: false,
            allowDecimals: false,
            value: 0,
            minValue: 0,
            maxValue: 100,
            listeners: {
                change: this.updateSlider.createDelegate(this)
            }
        });

        this.valueSlider = new Ext.Slider({
            height: 150,
            minValue: 0,
            increment: 1,
            maxValue: 100,
            vertical: true,
            plugins: new Ext.slider.Tip({
                getText: function(thumb) {
                    return String.format('{0}%', thumb.value);
                }
            }),
            listeners: {
                changeComplete: this.handleSliderChanged.createDelegate(this)
            }
        });

        this.stopButton = new Ext.Button({
            x: 120,
            y: 140,
            xtype: 'button',
            text: dSS._("Stop"),
            width: 50,
            handler: this.handleStop.createDelegate(this)
        });

        Ext.apply(this, {
            layout: 'absolute',
            padding: 5,
            height: 176,
            items: 
            [ 
                {
                    x: 10,
                    y: 16,
                    xtype: 'label',
                    text: dSS._("Value") + ':'
                },
                this.valueField,
                {
                    x: 175,
                    y: 16,
                    xtype: 'label',
                    text: '%'
                },
                {
                    x: 190,
                    y: 12,
                    xtype: 'panel',
                    border: false,
                    layout: 'form',
                    items: [ this.valueSlider ]
                },
                {
                    xtype: 'button',
                    x: 222,
                    y: 12,
                    text: dSS._("Max"),
                    tooltip: dSS._("Set maximum value"),
                    width: 50,
                    handler: this.handleMax.createDelegate(this)
                },
                {
                    x: 222,
                    y: 54,
                    xtype: 'button',
                    iconCls: 'buttonUpIcon',
                    tooltip: dSS._("Increase value"),
                    width: 50,
                    handler: this.handleInc.createDelegate(this)
                },
                {
                    x: 222,
                    y: 98,
                    xtype: 'button',
                    iconCls: 'buttonDownIcon',
                    tooltip: dSS._("Decrease value"),
                    width: 50,
                    handler: this.handleDec.createDelegate(this)
                },
                this.stopButton,
                {
                    x: 222,
                    y: 140,
                    xtype: 'button',
                    text: dSS._("Min"),
                    tooltip: dSS._("Set minimum value"),
                    width: 50,
                    handler: this.handleMin.createDelegate(this)             
                }
            ]
        });
        
        this.setData(this.dsid, config.group);

        dSS.devices.DeviceSettingsPanel.superclass.constructor.apply(this, arguments);
    },
    percentToValue: function(percent) {
        var value = percent / 100;
        value = value * 255;
        if (value < 0) {
            value = 0;
        } else if (value > 255) {
            value = 255;
        }
        return Math.round(value);
    },
    handleSliderChanged: function(slider, newVal, thumb) {
        this.valueField.setValue(newVal);
        dSS.util.setValueOnDevice(this.dsid, this.percentToValue(newVal));
    },
    updateSlider: function(field, newVal, oldVal) {
        if (newVal !== oldVal) {
            this.valueSlider.setValue(newVal);
            dSS.util.setValueOnDevice(this.dsid, this.percentToValue(newVal));
        }
    },
    handleMax: function() {
        dSS.util.callScene(this.dsid, dSS.config.scenes.MAX);
        this.valueField.setValue(100);
        this.valueSlider.setValue(100);
    },
    handleMin: function() {
        dSS.util.callScene(this.dsid, dSS.config.scenes.MIN);
        this.valueField.setValue(0);
        this.valueSlider.setValue(0);
    },
    handleStop: function() {
        dSS.util.callScene(this.dsid, dSS.config.scenes.STOP);
    },
    handleInc: function() {
        dSS.util.callScene(this.dsid, dSS.config.scenes.INC);
    },
    handleDec: function() {
        dSS.util.callScene(this.dsid, dSS.config.scenes.DEC);
    },
    setData: function(dsid, group) {
        this.dsid = dsid;
        if ((group === dSS.config.groups.SHADOW) ||
            (group === dSS.config.groups.ACCESS) ||
            (group === dSS.config.groups.SECURITY)) {
            this.stopButton.setVisible(true);
        } else {
            this.stopButton.setVisible(false);
        }
        this.doLayout();
        this.valueField.setValue(0);
        this.valueSlider.setValue(0);
    }
});

Ext.reg('dssdevsettingspanel', dSS.devices.DeviceSettingsPanel);

// parameters:
//  maskElement
//  device record from the device store
dSS.devices.DeviceSettingsWindow = Ext.extend(Ext.Window, {
    constructor: function(config) {
        if ((config.devices === undefined) || (config.index === undefined) ||
            (config.devices.length === 0)) {
            Ext.Msg.alert(dSS._("Error"), dSS._("No information for device"));
            return;
        }

        if (config.rowSel !== undefined) {
            this.rowSel = config.rowSel;
        }

        var disableLeft = true;
        var disableRight = true;

        this.devices = config.devices;
        this.index = config.index;

        var device = this.devices[this.index];

        if (this.devices.length > 1) {
            if (this.index > 0) {
                disableLeft = false;
            }
            if (this.index < this.devices.length - 1) {
                disableRight = false;
            }
        }
  
        var devInfo = dSS.util.parseProductID(device.get('productID'));
        var devCls = dSS.util.parseFunctionID(device.get('functionID'));

        this.left = new Ext.Button({
            iconCls: 'buttonLeftIcon',
            tooltip: dSS._("Select previous device"),
            width: 50,
            disabled: disableLeft,
            handler: this.previousDevice.createDelegate(this)
        });

        this.right = new Ext.Button({
            iconCls: 'buttonRightIcon',
            tooltip: dSS._("Select next device"),
            width: 50,
            disabled: disableRight,
            handler: this.nextDevice.createDelegate(this)
        });

        this.cutWidth = 286;

        Ext.apply(this, {
            layout: 'table',
            layoutConfig: {
                columns: 1,
                tableAttrs: {
                    width: '100%'
                }
            },
            padding: 10,
            width: 320,
            iconCls: dSS.util.getDeviceIconCSS(devInfo[0], devCls[0], true),
            resizable: false,
            title: dSS._("Device settings"),
            items: 
            [
                {
                    xtype: 'panel',
                    ref: 'info',
                    border: false,
                    items: 
                    [ 
                        dSS.util.CreateDefaultDeviceInfoPanel(
                                                            device.get('name'),
                                                            device.get('dsid'),
                                                            this.cutWidth)
                    ]
                },
                {
                
                    xtype: 'dssdevsettingspanel',
                    ref: 'settings',
                    dsid: device.get('dsid'),
                    group: devCls[0]
                
                }
            ],
            buttonAlign: 'left',
            buttons: 
            [
                this.left,
                this.right,
                new Ext.Toolbar.Fill(),
                {
                    xtype: 'button',
                    text: dSS._("Close"),
                    scope: this,
                    handler: function() {
                        this.maskElement.unmask();
                        this.close();
                    }
                }
            ]
        });

        dSS.devices.DeviceSettingsWindow.superclass.constructor.apply(this,
                                                                     arguments);

        this.maskElement.mask();

        this.on("beforedestroy", function(component) {
            this.maskElement.unmask();
        },
        this);
    },
    updateView: function() {
        var device = this.devices[this.index];
        var devInfo = dSS.util.parseProductID(device.get('productID'));
        var devCls = dSS.util.parseFunctionID(device.get('functionID'));
        this.setIconClass(dSS.util.getDeviceIconCSS(devInfo[0], devCls[0], true));

        this.info.removeAll(true);
        this.info.add(
                dSS.util.CreateDefaultDeviceInfoPanel(device.get('name'),
                                                      device.get('dsid'),
                                                      this.cutWidth));
        this.info.doLayout();
        this.settings.setData(device.get('dsid'), devCls[0]);
        if (this.rowSel !== undefined) {
            this.rowSel.selectRow(this.index, false, false);
        }
    },
    previousDevice: function() {
        if (this.index > 0) {
            this.index--;
            if (this.index == 0) {
                this.left.disable();
            }

            if ((this.devices.length > 1) && 
                (this.index < (this.devices.length - 1))) {
                this.right.enable();
            }
        }
        this.updateView();
    },
    nextDevice: function() {
        if (this.index < (this.devices.length - 1)) {
            this.index++;
            if (this.index === (this.devices.length - 1)) {
                this.right.disable();
            }

            if ((this.devices.length > 1) && (this.index > 0)) {
                this.left.enable();
            }
        }
        this.updateView();
    }
});

