Ext.namespace("dSS", "dSS.config", "dSS.devprops");

// form *must* react to the submit event and fire a formsubmitted
// event in response; it *must* also fire a formloaded event once all d
// has been received
//
// parameters: dsid
dSS.devprops.OutputModeForm = Ext.extend(Ext.form.FormPanel, {
    constructor: function(config) {
        if (config.dsid === undefined) {
            Ext.MessageBox.alert(dSS._("Error"), ("Missing dSID"));
            return;
        }

        this.dsid = config.dsid;

        this.addEvents('formsubmitted', 'formloaded');

        Ext.apply(this, {
            bubbleEvents: [ 'formsubmitted', 'formloaded' ],
            padding: 5,
            scope: this,
            params: { 'class' : 3, index : 0, dsid : config.dsid },
            items:
            [
                {
                    xtype: 'radiogroup',
                    ref: 'radio',
                    fieldLabel: dSS._("Output mode"),
                    columns: 1,
                    items:
                    [
                        {
                            boxLabel: dSS._("switch"),
                            name: 'rb_outmode_' + config.dsid,
                            id: dSS.config.deviceParams.OUTPUT_MODE_SWITCH.toString(),
                            inputValue : dSS.config.deviceParams.OUTPUT_MODE_SWITCH
                        },
                        {
                            boxLabel: dSS._("dimmer"),
                            name: 'rb_outmode_' + config.dsid,
                            id: dSS.config.deviceParams.OUTPUT_MODE_DIMMER.toString(),
                            inputValue : dSS.config.deviceParams.OUTPUT_MODE_DIMMER
                        },
                        {
                            boxLabel: dSS._("disabled"),
                            name: 'rb_outmode_' + config.dsid,
                            id: dSS.config.deviceParams.OUTPUT_MODE_DISABLED.toString(),
                            inputValue : dSS.config.deviceParams.OUTPUT_MODE_DISABLED
                        }
                    ]
                }
            ]
        });

        dSS.devprops.OutputModeForm.superclass.constructor.apply(this, arguments);
        // if masking is done in afterrender, the animation is not positioned
        // correctly; however we can not do the ajax call in afterlayout,
        // because it seems that afterlayout gets called twice
        this.on('afterlayout', function() {
                this.el.mask(dSS._("Loading output mode settings") + "...",
                             'x-mask-loading');
            },
            this);

        this.on('afterrender', function() {
            this.AjaxRequest = Ext.Ajax.request(
                {
                    url: '/json/device/getConfig',
                    method: 'GET',
                    params:
                    {
                        'class' : 3,
                        index : 0,
                        dsid : this.dsid
                    },
                    scope: this,
                    success: function(response) {
                        this.AjaxRequest = undefined;

                        if (this.closed === true) {
                            return;
                        }

                        try {
                            var jsonData = Ext.util.JSON.decode(
                                                response.responseText);
                            if (jsonData.ok !== true) {
                                this.el.unmask();
                                this.fireEvent('formloaded', true);
                                return;
                            }

                            if (this.closed === true) {
                                return;
                            }

                            var val = jsonData.result.value;
                            this.radio.setValue(val, val);
                            this.loadedValue = val;
                            this.el.unmask();
                            this.fireEvent('formloaded', true);

                        } catch (error) {
                            if (this.closed === true) {
                                return;
                            }

                            Ext.Msg.alert(dSS._("Error"), error);
                            this.el.unmask();
                            this.fireEvent('formloaded', true);
                        }
                    },
                    failure: function(result, request) {
                        this.AjaxRequest = undefined;
                        if (this.closed === true) {
                            return;
                        }

                        Ext.Msg.alert(dSS._("Error"),
                             dSS._("Could not load output mode settings"));
                        this.radio.markInvalid(
                             ("Current settings could not be loaded"));
                        this.el.unmask();
                        this.fireEvent('formloaded', true);
                                            }
                });
            },
            this);

        this.on('beforedestroy', function() {
                if (this.AjaxRequest !== undefined) {
                    Ext.Ajax.abort([this.AjaxRequest.tId]);
                }
                this.closed = true;
            },
            this);

        this.on('submit', function() {
            var value = this.radio.getValue().getRawValue();

            if (value == this.loadedValue) {
                this.fireEvent('formsubmitted');
                return;
            }

            // this should never happen, but lets double check so we
            // do not attempt to write some random crap into the klemme
            if ((value !== dSS.config.deviceParams.OUTPUT_MODE_SWITCH.toString()) &&
                (value !== dSS.config.deviceParams.OUTPUT_MODE_DISABLED.toString()) &&
                (value !== dSS.config.deviceParams.OUTPUT_MODE_DIMMER.toString())) {
                Ext.Msg.alert(dSS._("Error"), dSS._("Attempted to set invalid value") +
                    ": " + value);
                this.fireEvent('formsubmitted');
                return;
            }

            this.AjaxRequest = dSS.util.genericRequest({
                url: '/json/device/setConfig',
                params:
                {
                    'class' : 3,
                    index : 0,
                    dsid : this.dsid,
                    value : value
                },
                failureMsg: dSS._("Could not save output mode settings"),
                maskEl: this.el,
                maskMsg: dSS._("Saving output mode settings"),
                scope: this,
                evt: 'formsubmitted'
            });
        },
        this);
    }
});

Ext.reg('dssdevoutputmodeform', dSS.devprops.OutputModeForm);

